#!/usr/bin/python
import sys

from charmhelpers.core.hookenv import (
    Hooks,
    config,
    is_relation_made,
    log,
    INFO,
    ERROR,
    local_unit,
    relation_ids,
    relation_get,
    relation_set,
    related_units,
    service_name,
    unit_get,
    UnregisteredHookError,
)
from charmhelpers.core.host import (
    lsb_release,
    restart_on_change,
    service_restart,
)

from charmhelpers.fetch import (
    add_source,
    apt_install,
    apt_purge,
    apt_update,
    apt_upgrade,
    filter_installed_packages,
)

from charmhelpers.payload.execd import execd_preinstall
from charmhelpers.contrib.charmsupport import nrpe

from prometheus_utils import (
    create_datadir,
    determine_packages,
    juju_log,
    restart_map,
    write_conf,
)

from socket import gethostname

hooks = Hooks()

def install_packages():
    add_source(config('source'))
    apt_update()
    apt_upgrade()
    apt_install(determine_packages(), fatal=True)

@hooks.hook()
def install():
    execd_preinstall()
    create_datadir()
    install_packages()
    write_conf()

@hooks.hook('start', 'stop')
def startstop():
    juju_log('start/stop triggered - noop for now')


@hooks.hook('target-relation-joined', 'target-relation-departed')
@restart_on_change(restart_map())
def update_targets():
    'Update related_targets if target:http relation exists'

    # targe-relation-{joined,changed,departed} triggered
    if not is_relation_made('target'):
        write_conf()
        return

    _targets = ' '
    # no relations found
    r_ids = relation_ids('target')
    if len(r_ids) == 0:
        write_conf()
        return

    for r_id in r_ids:
        # loop per related unit
        for r_unit in related_units(r_id):
            settings = relation_get(unit=r_unit, rid=r_id)
            log('settings(rid=%s)=> %s' % (str(r_id), settings))
            if settings and 'host' in settings and 'port' in settings:
                _targets += '%s:%s,' % (settings['host'], str(settings['port']))
    if _targets != ' ':
        write_conf(_targets[:-1])
    # not host:port data found
    else:
        write_conf()


@hooks.hook('config-changed', 'target-relation-changed')
@restart_on_change(restart_map())
def config_changed():
    install_packages()
    update_targets()
    service_restart('prometheus')


@hooks.hook('upgrade-charm')
def upgrade_charm():
    config_changed()


def main():
    try:
        hooks.execute(sys.argv)
    except UnregisteredHookError as e:
        log('Unknown hook {} - skipping.'.format(e))


if __name__ == '__main__':
    main()
