import os
import shutil
import pwd

from base64 import b64decode
from copy import deepcopy
from subprocess import check_call, check_output, CalledProcessError

from charmhelpers.fetch import (
    add_source,
    apt_update,
    apt_upgrade,
    apt_install,
)

from charmhelpers.core.host import (
    service_restart,
    service_start,
    service_stop,
    lsb_release,
    write_file,
)

from charmhelpers.core.hookenv import (
    charm_dir,
    config,
    log,
    related_units,
    relation_ids,
    relation_get,
    DEBUG,
    INFO,
)

from charmhelpers.core.templating import render
#from charmhelpers.core.decorators import retry_on_exception

from charmhelpers.contrib.python.packages import (
    pip_install,
)

from prometheus_contexts import (
    CollectdExporterContext,
    sanitize_targets,
)

TEMPLATES = 'templates/'

BASE_PACKAGES = [
    'prometheus',
]

PROM_CONF_DIR = "/etc/prometheus"
PROM_CONF = '%s/prometheus.yml' % PROM_CONF_DIR
PROM_DATA_DIR = '/var/lib/prometheus'


BASE_RESOURCE_MAP = {
    PROM_CONF: {
        'services': ['prometheus'],
        'contexts': [CollectdExporterContext()]
    },
}

def create_datadir(datadir=PROM_DATA_DIR):
    users = [i for i in pwd.getpwall() if i.pw_name == 'nobody']
    if not os.path.exists(datadir):
        os.mkdir(datadir)

    if len(users) == 1:
        os.lchown(datadir, users[0].pw_uid, users[0].pw_gid)


def juju_log(msg):
    log('[prometheus] %s' % msg)

def resource_map():
    '''
    Dynamically generate a map of resources that will be managed for a single
    hook execution.
    '''
    resource_map = deepcopy(BASE_RESOURCE_MAP)

    return resource_map


def restart_map():
    '''
    Constructs a restart map based on charm config settings and relation
    state.
    '''
    return {k: v['services'] for k, v in resource_map().iteritems()}


def services():
    ''' Returns a list of services associate with this charm '''
    _services = []
    for v in restart_map().values():
        _services = _services + v
    return list(set(_services))


def write_conf(targets=None):
    '''
    Renders templates
    '''
    for cfg, d in resource_map().iteritems():
        tmpl = '%s.tmpl' % os.path.basename(cfg)
        ctxt_dict = {}
        for ctxt_obj in d['contexts']:
            ctxt_dict.update(ctxt_obj())
        if targets:
            ctxt_dict['related_targets'] = sanitize_targets(targets)
        juju_log('ctxt_dict: %s' % str(ctxt_dict))
        render(tmpl, cfg, ctxt_dict, templates_dir=TEMPLATES)


def update_static_targets(targets):
    for cfg, d in resource_map().iteritems():
        if 'prometheus' in d['services']:
            for k, v in enumerate(d['contexts']):
                if type(k) == type(CollectdExporterContext):
                    d['contexts'][k] = CollectdExporterContext()


def determine_packages():
    packages = [] + BASE_PACKAGES

    return packages

