import prometheus_contexts as contexts
import os

os.environ['JUJU_UNIT_NAME'] = 'prometheus'
import prometheus_utils as utils

from mock import patch, MagicMock

from test_utils import (
    CharmTestCase
)

TO_PATCH = [
    'config',
    'relation_ids',
    'service_name',
    'related_units',
    'relation_get',
]


class TestPrometheusContext(CharmTestCase):

    def setUp(self):
        super(TestPrometheusContext, self).setUp(contexts, TO_PATCH)

    def test_collectd_exporter_related(self):
        self.relation_ids.return_value = ['collectd-exporter:0']
        self.config.return_value = '1'
        service = 'mycollectdexporter'
        self.service_name.return_value = service
        self.assertEquals(contexts.CollectdExporterContext()(),
			{'static_targets': "['1']",
			 'rules': '1',
			 'scrape_interval': '1',
			 'evaluation_interval': '1'
			})

    def test_sanitize_targets(self):
        'Concert yaml config targets into prometheus yaml format'
        ex_result = "['127.0.0.1:9090', '10.0.0.1:9090']"
        ex_list = ['127.0.0.1:9090', '10.0.0.1:9090']
        ex_str = '127.0.0.1:9090, 10.0.0.1:9090'
        ex_ustr = u'127.0.0.1:9090, 10.0.0.1:9090'

        self.assertEquals(contexts.sanitize_targets(ex_list),
		ex_result)
        self.assertEquals(contexts.sanitize_targets(ex_str),
		ex_result)
        self.assertEquals(contexts.sanitize_targets(ex_ustr),
		ex_result)
